// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008

/**
 * A utility class for logging within the framework
 */
package com.ibm.websphere.batch.devframework.configuration;

import java.util.Properties;
/**
 * 
 * @author Neeraj
 * This class is used by the BDS Framework to log INFO, DEBUG and ERROR messages to system out
 * Logging is enabled by passing the property debug=true in the step or stream properties.
 * By default logging is disabled.
 */
public class BDSFWLogger {

	protected String jobId = "default_job_id";
	protected boolean isDebugEnabled = false;
	
	
	/**
	 * The constructor looks for a property named debug to enable or disable logging
	 * @param props
	 */
	
	public BDSFWLogger (Properties props)
	{
		String debugEnabled = props.getProperty(BDSFrameworkConstants.debugEnabledKey);
		if (debugEnabled != null)
		{
			if (debugEnabled.trim().toLowerCase().equals("true"))
			{
				isDebugEnabled = true;				
			}
			else
			{
				isDebugEnabled = false;							
			}
		}
		
		String jobIdVal = props.getProperty(BDSFrameworkConstants.jobIDKey);
		if (jobIdVal != null)
		{
			jobId = jobIdVal;
		}
		else 
		{
			System.out.println("BDSFWLogger- jobID is null!");
		}
						
	}
	
	public void setDebug(boolean debug)
	{
		this.isDebugEnabled = debug;
	}
	
	public boolean isDebugEnabled()
	{
		return isDebugEnabled;
	}
	
	public void debug(String message)
	{
		System.out.println("DEBUG->jobid: " + jobId + ":" + message);
	}
	
	public void info(String message)
	{
		System.out.println("INFO->jobid: " + jobId + ":" + message);
	}
	
	public void error(String message)
	{
		System.out.println("ERROR->jobid: " + jobId + ":" + message);
	}
	
}
