// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008

package com.ibm.websphere.batch.devframework.datastreams.patternadapter;


import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

import com.ibm.websphere.batch.BatchContainerDataStreamException;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.configuration.PerformanceAnalyzer;
import com.ibm.websphere.batch.devframework.datastreams.bdsadapter.AbstractBatchDataInputStream;
import com.ibm.websphere.batch.devframework.datastreams.bdsadapter.AbstractBatchDataStream;

public abstract class BDSJPAReader extends AbstractBatchDataStream 
implements AbstractBatchDataInputStream 
{
	public static final String CLASSNAME = BDSJPAReader.class.getName();
	
	
	protected JPAReaderPattern jpaReaderImpl; 
	
	protected Object record;
	protected String restartToken;
	protected BDSFWLogger logger;
	private EntityManagerFactory _entityManagerFactory = null;
	private EntityManager _entityManager = null;
	private Iterator _resultListIt = null;
	
	protected PerformanceAnalyzer perfAnalyzer = null;
	protected boolean isPerformanceMeasurementEnabled = false;
	public void initialize(Properties props)
	{
		try 
		{
			logger = new BDSFWLogger(props);
			perfAnalyzer = new PerformanceAnalyzer(CLASSNAME,props);
			isPerformanceMeasurementEnabled = perfAnalyzer.isEnabled();
			String persistentUnit = 
				getRequiredProperty(props,BDSFrameworkConstants.PERSISTENT_UNIT_KEY);
			
			if(_entityManagerFactory == null)
			{	
				_entityManagerFactory = 
					Persistence.createEntityManagerFactory(persistentUnit, props);
			}	
			String jpaReaderImplClass = props.getProperty(BDSFrameworkConstants.IMPLCLASS_KEY);
			// For backward compatibility
			if(jpaReaderImplClass == null)
				jpaReaderImplClass = getRequiredProperty(props,"IMPLCLASS");	
			jpaReaderImpl = (JPAReaderPattern) loadClass(jpaReaderImplClass);
			if(logger.isDebugEnabled())
				logger.info("Loaded JPAReaderPattern "  + jpaReaderImplClass);
			jpaReaderImpl.initialize(props);
			
		}
		catch (Throwable t)
		{
			throw new RuntimeException(t);
		}
	}
	
	

	public void positionAtInitialCheckpoint() throws BatchContainerDataStreamException {
		String methodName = "positionAtInitialCheckpoint";
		if(logger.isDebugEnabled()) 
			logger.debug("Entering " + CLASSNAME + ":" + methodName);
		
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		
		try 
		{			
			String query = jpaReaderImpl.getInitialLookupQuery();
			if(logger.isDebugEnabled()) 
				logger.debug("Initial lookup query" + query);
			_createQueryAndExecute(query);
		}
		catch (Throwable t)
		{
			throw new BatchContainerDataStreamException(t);
		}		
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
		if(logger.isDebugEnabled()) 
			logger.debug("Exiting " + CLASSNAME + ":" + methodName);
	}
	
	private void _createQueryAndExecute(String query) {
				
		Query q = _entityManager.createQuery(query);
		List resultList = q.getResultList();
		_resultListIt = resultList.iterator();
	
		
	}
	
	public void positionAtCurrentCheckpoint()
			throws BatchContainerDataStreamException {
		String methodName = "positionAtCurrentCheckpoint";
		if(logger.isDebugEnabled()) 
			logger.debug("Entering " + CLASSNAME + ":" + methodName);
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		
		try 
		{
			String query = jpaReaderImpl.getRestartQuery(restartToken);
			if(logger.isDebugEnabled()) 
				logger.debug("Restart query" + query);
			_createQueryAndExecute(query);

		}
		catch (Throwable t)
		{
			throw new BatchContainerDataStreamException(t);
		}
		
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
		if(logger.isDebugEnabled()) 
			logger.debug("Exiting " + CLASSNAME + ":" + methodName);
	}

	public Object read() throws Exception {
		String methodName = "read";
		if(logger.isDebugEnabled()) 
			logger.debug("Entering " + CLASSNAME + ":" + methodName);
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		
		Object rec = jpaReaderImpl.fetchRecord(_resultListIt); 
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
		if(logger.isDebugEnabled()) 
			logger.debug("Exiting " + CLASSNAME + ":" + methodName + "record:" + rec);
		return rec;
	}
	
	public Object read(Object pql) throws Exception {
		String methodName = "read";
		if(logger.isDebugEnabled()) 
			logger.debug("Entering " + CLASSNAME + ":" + methodName);
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		
		Query query  = _entityManager.createQuery((String)pql);
		Object result = query.getSingleResult();
		
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
		if(logger.isDebugEnabled()) 
			logger.debug("Exiting " + CLASSNAME + ":" + methodName);
		return result;
	}

	public boolean hasNext() throws Exception {
		return _resultListIt.hasNext();
	}
	
	public void open() throws BatchContainerDataStreamException {
		String methodName = "open";
		if(logger.isDebugEnabled()) 
			logger.debug("Entering " + CLASSNAME + ":" + methodName);
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		_entityManager = this._entityManagerFactory.createEntityManager();
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
		if(logger.isDebugEnabled()) 
			logger.debug("Exiting " + CLASSNAME + ":" + methodName);
	}

	public void close() throws BatchContainerDataStreamException {
		String methodName = "close";
		if(logger.isDebugEnabled()) 
			logger.debug("Entering " + CLASSNAME + ":" + methodName);

		try {			
			_entityManager.close();
			_entityManagerFactory.close();
			if(isPerformanceMeasurementEnabled)
				perfAnalyzer.displayPerformanceStatisticsForEntity();

		} catch (Throwable t) {
			throw new BatchContainerDataStreamException(t);
		}
		if(logger.isDebugEnabled()) 
			logger.debug("Exiting " + CLASSNAME + ":" + methodName);
	}

	public String externalizeCheckpointInformation() {
		String methodName = "externalizeCheckpointInformation";
		if(logger.isDebugEnabled()) 
			logger.debug("Entering " + CLASSNAME + ":" + methodName);
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		
		try
		{			
			String restartToken = jpaReaderImpl.getRestartTokens();	
			if(isPerformanceMeasurementEnabled)
				perfAnalyzer.endMeasurementAndSave(methodName);
			if(logger.isDebugEnabled()) 
				logger.debug("Exiting " + CLASSNAME + ":" + methodName + restartToken);
			return restartToken;
		}
		catch (Throwable t)
		{
			throw new RuntimeException(t);
		}
		
		
	}
	
	public void internalizeCheckpointInformation(String token) throws RuntimeException {
		String methodName = "internalizeCheckpointInformation";
		
		if (logger.isDebugEnabled())
			logger.debug(CLASSNAME +"." + methodName + "- token= |" + token + "|");
		
			restartToken = token;
			
		if(logger.isDebugEnabled()) 
			logger.debug("Exiting " + CLASSNAME + ":" + methodName);		
	}
	
	public Object fetchHeader() {
		// NOT APPLICABLE
		return null;
	}



	
}