// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008

package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * 
 * @author Snehal/Neeraj
 * 
 * This pattern is used to implement the business logic of processing a byte
 * input stream. The administrative tasks of opening, closing the stream
 * is taken care of by the batch framework.
 * 
 * Prequisites:
 * - Needs the following properties to be defined in the xJCL
 * 1.  <prop name="FILENAME" value="/opt/tmp/input.vb.txt" />
 * (Optional)
 * 1.  <prop name="debug" value="true"/>
 */

public interface ByteReaderPattern {

	/**
	 * Is called by the framework during Step setup stage
	 * @param props
	 */
	public void initialize(Properties props);
	
	/**
	 * 
	 * @param reader
	 * @throws IOException
	 */
	
	public void processHeader(BufferedInputStream reader) throws IOException;
	
	/**
	 * Get the next record from the input stream
	 * @param reader
	 * @return
	 * @throws IOException
	 */
	public Object fetchRecord(BufferedInputStream reader) throws IOException;
	
	/**
	 * This method can be optionally invoked from the process step
	 * to obtain the header data that was previously obtained during the processHeader
	 * call
	 * @return
	 */
	
	public Object fetchHeader();
}
