// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
/**
 * 
 * @author Snehal/Neeraj
 * 
 * This pattern is used to implement the business logic of processing a byte
 * output stream. The administrative tasks of opening, closing the stream
 * is taken care of by the batch framework.
 * 
 * Prerequisites:
 * - Needs the following properties to be defined in the xJCL
 * 1.  <prop name="FILENAME" value="/opt/tmp/input.vb.txt" />
 * (Optional)
 * 1.  <prop name="debug" value="true"/>
 */

package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.io.BufferedOutputStream;
import java.io.IOException;
import java.util.Properties;

public interface ByteWriterPattern {

	/**
	 * Invoked during the step setup phase
	 * @param props
	 */
	public void initialize(Properties props);
	
	/**
	 * Writes the given object onto the given outputstream. Any processing
	 * that needs to be done before writing can be added here 
	 * @param out
	 * @param record
	 * @throws IOException
	 */
	public void writeRecord(BufferedOutputStream out, Object record) throws IOException;
	
	/**
	 * Write header information if any
	 * @param out
	 * @throws IOException
	 */
	public void writeHeader(BufferedOutputStream out) throws IOException;
	
	/**
	 * This method can be optionally called during process step to explicity 
	 * initialize and write the header. 
	 * @param header
	 */
	public void writeHeader(BufferedOutputStream out, Object header) throws IOException;
}
