// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008


/**
 * 
 * @author Snehal/Neeraj
 * 
 * This pattern is used to implement the business logic of processing a text
 * input stream. The administrative tasks of opening, closing the stream
 * is taken care of by the batch framework.
 * 
 * Prequisites:
 * - Needs the following properties to be defined in the xJCL
 * 1.  <prop name="FILENAME" value="/opt/tmp/input.vb.txt" />
 * (Optional)
 * 1.  <prop name="debug" value="true"/>
 * 2.  <prop name="file_encoding" value="CP1047"/>
 */
package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.Properties;



public interface FileReaderPattern {

	/**
	 * Invoked during the step setup phase
	 * @param props
	 */
	public void initialize(Properties props);
	/**
	 * This method is invoked only once. It should be used
	 * to write any header data if necessary.
	 * @param reader
	 * @throws IOException
	 */
	public void processHeader(BufferedReader reader) throws IOException;
	
	/**
	 * This method should read the next line from the reader
	 * and return the data in suitable form to be processed
	 * by the step.
	 * @param reader
	 * @return
	 * @throws IOException
	 */
	public Object fetchRecord(BufferedReader reader) throws IOException;
	
	/**
	 * This method can be optionally invoked from the process step
	 * to obtain the header data that was previously obtained during the processHeader
	 * call
	 * @return
	 */
	
	public Object fetchHeader();
}
