// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008


/**
 * 
 * @author Snehal/Neeraj
 * 
 * This pattern is used to implement the business logic of processing a text
 * input stream. The administrative tasks of opening, closing the stream
 * is taken care of by the batch framework.
 * 
 * Prequisites:
 * - Needs the following properties to be defined in the xJCL
 * 1.  <prop name="FILENAME" value="/opt/tmp/output.vb.txt" />
 * (Optional)
 * 1.  <prop name="debug" value="true"/>
 * 2.  <prop name="file_encoding" value="CP1047"/>
 */
package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Properties;


public interface FileWriterPattern {

	/**
	 * Invoked during step setup phase
	 * @param props
	 */
	public void initialize(Properties props);
	
	/**
	 * This method should write the given record
	 * object to the bufferedwriter.
	 * @param out
	 * @param record
	 * @throws IOException
	 */
	public void writeRecord(BufferedWriter out, Object record) throws IOException;
	
	/**
	 * This method is invoked only once just after the bufferedwriter
	 * is opened. It should be used to write any header information
	 * @param out
	 * @throws IOException
	 */
	public void writeHeader(BufferedWriter out) throws IOException;
	
	/**
	 * This method can be optionally called during process step to explicity 
	 * initialize and write the header. 
	 * @param header
	 * @throws IOException 
	 */
	public void writeHeader(BufferedWriter out, Object header) throws IOException;
}
