//Licensed Materials - Property of IBM 
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
package com.ibm.websphere.batch.devframework.datastreams.patternadapter;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;


/**
* 
* @author Neeraj
* 
* This pattern is used to implement the business logic of processing data from
*  JDBC using a Stored Procedure. The administrative tasks of opening, closing the connection
* is taken care of by the batch framework.
* 
* Prequisites:
* 
* E.g
* 
* 
*  
* For a Network JDBC database:
* - Needs the following properties to be defined in the xJCL
* ds_jndi_name, tablename
* 
* E.g
* <bds>
*			<logical-name>inputStream</logical-name>
*			<props>
*				<prop name="IMPLCLASS" value="com.ibm.websphere.batch.samples.creditsample.MYDBReader"/>
*              <prop name="ds_jndi_name" value="jdbc/crreport"/>
*              <prop name="debug" value="true"/>
*			</props>
*			<impl-class>com.ibm.websphere.batch.devframework.datastreams.patterns.JDBCCallableStatementReader</impl-class>
* </bds>
* 
*/

public interface JDBCCallableStatementReaderPattern {


	/**
	 * This method is invoked during the job setup phase. 
	 * 
	 * @param props properties provided in the xJCL 
	 */
	
	public void initialize(Properties props);
	
	/**
	 * This method should retrieve values for the various columns for the current row from the given resultset object.
	 * Typically this data would be used to populate an intermediate object which would be returned
	 * @param resultSet
	 * @return
	 */
	public Object fetchRecord(ResultSet resultSet);
	
	/**
	 * This method should return a ResultSet used by the stream to retrieve all relevant
	 * data that would be processed part of the job steps.
	 */
	public ResultSet getResultSet(CallableStatement cstmt) throws SQLException ;

	/**
	 * This method should return a Callable statement that will be used during setup of the stream to retrieve all relevant
	 * data that would be processed part of the job steps
	 * @return object to be used during process step.
	 */
	public CallableStatement getInitialCallableStatement(Connection con);
	
	/**
	 * This method gets called during Job Restart. The restart token should be used to create a Callable Statement that will
	 * retrieve previously unprocessed records. 
	 * Typically the restart token would be the primary key in the table and the query would get all rows with 
	 * primary key value > restarttoken
	 * @param restartToken
	 * @return The restart query
	 */
	public CallableStatement getRestartCallableStatement(Connection con,String restartToken);
	
	/**
	 * This method gets called just before a checkpoint is taken.
	 * @return The method should return a string value identifying the last record read by the stream.
	 */
	public String getRestartTokens();
		
}
