
//Licensed Materials - Property of IBM 
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008


/**
* 
* @author Neeraj
* 
* This pattern is used to implement the business logic of outputing data to
*  JDBC using a Stored Procedure. The administrative tasks of opening, closing the connection
* is taken care of by the batch framework.
* 
* Prequisites:
* 
* For a Network JDBC database:
* - Needs the following properties to be defined in the xJCL
* ds_jndi_name, tablename
* 
* E.g
* <bds>
*			<logical-name>inputStream</logical-name>
*			<props>
*				<prop name="IMPLCLASS" value="com.ibm.websphere.batch.samples.creditsample.MYDBWriter"/>
*              <prop name="ds_jndi_name" value="jdbc/crreport"/>
*              <prop name="debug" value="true"/>
*			</props>
*			<impl-class>com.ibm.websphere.batch.devframework.datastreams.patterns.CallableStatementJDBCWriter</impl-class>
* </bds>
* 
*/
package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.sql.CallableStatement;
import java.util.Properties;

public interface JDBCCallableStatementWriterPattern {

	public void initialize(Properties props);
	
	/**
	 * This is typically an Update query used to write data into the DB
	 * @return
	 */
	public String getCallableStatementSQL();
	
	/**
	 * The parent class BDSCallableStatementJDBCWriter creates a new CallableStatement and
	 * passes it to this method. This method populates the CallableStatement
	 * with appropriate values and returns it to the parent class for execution
	 * @param pstmt
	 * @param record
	 * @return
	 */
	public CallableStatement writeRecord(CallableStatement pstmt, Object record);
}
