// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008

package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;

import javax.naming.InitialContext;
import javax.sql.DataSource;

import com.ibm.websphere.batch.BatchContainerDataStreamException;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;

public class JDBCConnection implements IJDBCConnection{
	
	private BDSFWLogger logger;
	private String ds_jndi_name;
	private boolean LPSEnabled = false;
	private boolean cursorHoldableEJBEnabled = false;
	private boolean autoCommitEnabled = true;
	private boolean useLocalJavaEnv = false;
	private DataSource ds = null;
	
	protected Connection conn;

	public void initialize(Properties props) {
		try 
		{
			logger = new BDSFWLogger(props);
			
			ds_jndi_name = getRequiredProperty(props,BDSFrameworkConstants.DSJNDINAME_KEY);
			
			String cursorHoldableEJBEnabledStr = props.getProperty(BDSFrameworkConstants.CURSOR_HOLDABILITY_ENABLED,"false");
			String lpsEnabled = props.getProperty(BDSFrameworkConstants.LPS_ENABLED,"false");
			String useLocalJavaEnvStr  = props.getProperty(BDSFrameworkConstants.USE_LOCAL_JAVA_ENV,"false");
			if(cursorHoldableEJBEnabledStr.equals("true"))
			{
				logger.info("Cursor Holdability enabled");
				cursorHoldableEJBEnabled = true;
			}
			if(useLocalJavaEnvStr.equals("true"))
			{
				logger.info("Using local Java namespace");
				useLocalJavaEnv = true;
			}
			if(lpsEnabled.equals("true")) {
				logger.info("LPS enabled");
				LPSEnabled = true;
			}
			if(cursorHoldableEJBEnabledStr.equals("true"))
			{
				logger.info("Cursor Holdability enabled");
				cursorHoldableEJBEnabled = true;
			}
			String autoCommitProp = props.getProperty(BDSFrameworkConstants.AUTO_COMMIT_ENABLED_KEY,"true");
			if(!autoCommitProp.equals("true")) {
				autoCommitEnabled = false;
			}
		}
		catch (Throwable t)
		{
			throw new RuntimeException(t);
		}

	}
	public void open() throws BatchContainerDataStreamException {
		try 
		{
			if (ds == null || conn == null || conn.isClosed()) {
				
				if(logger.isDebugEnabled())
					logger.debug("Loading datasource with JNDI name = " + ds_jndi_name);
				
				InitialContext ic = new InitialContext();
				if(cursorHoldableEJBEnabled || LPSEnabled)
				{
					ds = (DataSource)ic.lookup("java:comp/env/" + ds_jndi_name);
			
					conn = ds.getConnection();
					conn.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
					
					try {
						conn.setHoldability(ResultSet.HOLD_CURSORS_OVER_COMMIT);
					} catch (SQLException e) {
						logger.debug("WARNING: Cursor holdability may not be set ");
					}
					logger.debug("Loaded datasource with cursor holdability and rc");
				} else 	if (useLocalJavaEnv){	
					ds = (DataSource)ic.lookup("java:comp/env/" + ds_jndi_name);
					conn = ds.getConnection();
				} else {
					ds = (DataSource)ic.lookup(ds_jndi_name);
					conn = ds.getConnection();
				}
				
				if(!autoCommitEnabled) {
					//disable auto commit 
					conn.setAutoCommit(false);
				}
				
			}
			
			
			if(logger.isDebugEnabled())
				logger.info("Loaded Datasource using JNDI" + ds_jndi_name);
		}
		catch (Throwable t)
		{
			throw new BatchContainerDataStreamException(t);
		}

	}
	public void close() throws BatchContainerDataStreamException {
		try {			
			if(conn != null)
				conn.close();
		} catch (Throwable t) {
			throw new BatchContainerDataStreamException(t);
		}

	}
	public Connection getJDBCConnection() {
		if(conn != null)
		{	
			return conn;
		}
		else
		{
			throw new RuntimeException("Connection not initialized!");
		}
	}
	
	 String getRequiredProperty(Properties props,String key) {
	        String value = props.getProperty(key);
	        if (value == null) {
	            throw new RuntimeException("Missing required property '" + key + "'");
	        }
	        value = value.trim();
	        if (value.length() == 0) {
	            throw new RuntimeException("Required property '" + key + "' has empty value");
	        }
	        return value;
	    }   
	public void commit() throws SQLException {
		
			conn.commit();
		
		
	}   


}
