// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008


/**
 * 
 * @author Snehal/Neeraj
 * 
 * This pattern is used to implement the business logic of outputing data to
 *  JDBC. The administrative tasks of opening, closing the connection
 * is taken care of by the batch framework.
 * 
 * Prequisites:
 * For a Local JDBC database:
 * - Needs the following properties to be defined in the xJCL
 * jdbc_url, jdbc_driver, tablename and optionally user_id,pswd
 * E.g
 *  <bds>
 *		<logical-name>outputStream</logical-name>
 *		<props>
 *			<prop name="PATTERN-IMPL-CLASS" value="com.ibm.websphere.batch.samples.creditsample.LocalDBWriter"/>
 *          <prop name="jdbc_url" value="jdbc:derby:C:\\mysample\\CREDITREPORT"/>
 *          <prop name="jdbc_driver" value="org.apache.derby.jdbc.EmbeddedDriver"/>
 *          
 *          <prop name="user_id" value="myid"/>
 *          <prop name="pswd" value="mypwd"/>
 *          <prop name="debug" value="true"/>
 *		</props>
 *			<impl-class>com.ibm.websphere.batch.devframework.datastreams.patterns.LocalJDBCWriter</impl-class>
 *		</bds>
 * 
 *  
 * For a Network JDBC database:
 * - Needs the following properties to be defined in the xJCL
 * ds_jndi_name, tablename
 * 
 * E.g
 * <bds>
 *			<logical-name>inputStream</logical-name>
 *			<props>
 *				<prop name="IMPLCLASS" value="com.ibm.websphere.batch.samples.creditsample.MYDBWriter"/>
 *              <prop name="ds_jndi_name" value="jdbc/crreport"/>
 *              <prop name="debug" value="true"/>
 *			</props>
 *			<impl-class>com.ibm.websphere.batch.devframework.datastreams.patterns.JDBCWriter</impl-class>
 * </bds>
 * 
 */
package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.sql.PreparedStatement;
import java.util.Properties;

public interface JDBCWriterPattern {

	public void initialize(Properties props);
	
	/**
	 * This is typically an Update query used to write data into the DB
	 * @return
	 */
	public String getSQLQuery();
	
	/**
	 * The parent class BDSJDBCWriter creates a new preparedstatement and
	 * passes it to this method. This method populates the preparedstatement
	 * with appropriate values and returns it to the parent class for execution
	 * @param pstmt
	 * @param record
	 * @return
	 */
	public PreparedStatement writeRecord(PreparedStatement pstmt, Object record);
}
