// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008

package com.ibm.websphere.batch.devframework.datastreams.patternadapter;


import java.sql.ResultSet;
import java.util.Iterator;
import java.util.Properties;

/**
 * 
 * @author Neeraj
 * 
 * This pattern is used to implement the business logic of processing data from a
 *  JPA connection. The administrative tasks of opening, closing the connection
 * is taken care of by the batch framework.
 * 
 * Prequisites:
 *
 * - Needs the following properties to be defined in the xJCL
 * PERSISTENT_UNIT, openjpa.ConnectionDriverName, openjpa.ConnectionURL, openjpa.ConnectionURL and 
 * optionally openjpa.ConnectionUserName,openjpa.ConnectionPassword
 * E.g
 *  <bds>
 *	   <prop name="openjpa.ConnectionDriverName" value="org.apache.derby.jdbc.EmbeddedDriver"/>
 *     <prop name="openjpa.ConnectionURL" value="jdbc:derby:/opt/tmp/hellojpadb;create=true"/>
 *     <prop name="openjpa.ConnectionUserName" value="" />
 *     <prop name="openjpa.ConnectionPassword" value="" />
 *     <prop name="openjpa.jdbc.SynchronizeMappings" value="buildSchema"/>
 *     <prop name="openjpa.Log" value="DefaultLevel=WARN,SQL=TRACE"/>
 *     <prop name="PERSISTENT_UNIT" value="hellojpa"/>
 *     <prop name="debug" value="true"/>
 *     <prop name="PATTERN_IMPL_CLASS" value="com.ibm.websphere.samples.JPAInputStream"/>
 *   </bds>
 *  
 * 
 * 
 */

public interface JPAReaderPattern {

	/**
	 * This method is invoked during the job setup phase. 
	 * 
	 * @param props properties provided in the xJCL 
	 */
	
	public void initialize(Properties props);
	
	/**
	 * This method should retrieve values for the various columns for the current row from the given resultset object.
	 * Typically this data would be used to populate an intermediate object which would be returned
	 * @param listIt
	 * @return
	 */
	public Object fetchRecord(Iterator listIt);
	
	/**
	 * This method should return a JPQL query that will be used during setup of the stream to retrieve all relevant
	 * data that would be processed part of the job steps
	 * @return object to be used during process step.
	 */
	public String getInitialLookupQuery();
	
	/**
	 * This method gets called during Job Restart. The restart token should be used to create an JPQL query that will
	 * retrieve previously unprocessed records. 
	 * Typically the restart token would be the primary key in the table and the query would get all rows with 
	 * primary key value > restarttoken
	 * @param restartToken
	 * @return The restart query
	 */
	public String getRestartQuery(String restartToken);
	
	/**
	 * This method gets called just before a checkpoint is taken.
	 * @return The method should return a string value identifying the last record read by the stream.
	 */
	public String getRestartTokens();
		
}
