// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008



package com.ibm.websphere.batch.devframework.datastreams.patternadapter;


import java.util.Properties;

import javax.persistence.EntityManager;


/**
 * 
 * @author Neeraj
 * 
 * This pattern is used to implement the business logic of writing data using
 *  JPA. The administrative tasks of opening, closing the connection
 * is taken care of by the batch framework.
 * 
 * Prequisites:
 * 
 * - Needs the following properties to be defined in the xJCL
 * PERSISTENT_UNIT, openjpa.ConnectionDriverName, openjpa.ConnectionURL, openjpa.ConnectionURL and 
 * optionally openjpa.ConnectionUserName,openjpa.ConnectionPassword
 * E.g
 *  <bds>
 *	   <prop name="openjpa.ConnectionDriverName" value="org.apache.derby.jdbc.EmbeddedDriver"/>
 *     <prop name="openjpa.ConnectionURL" value="jdbc:derby:/opt/tmp/hellojpadb;create=true"/>
 *     <prop name="openjpa.ConnectionUserName" value="" />
 *     <prop name="openjpa.ConnectionPassword" value="" />
 *     <prop name="openjpa.jdbc.SynchronizeMappings" value="buildSchema"/>
 *     <prop name="openjpa.Log" value="DefaultLevel=WARN,SQL=TRACE"/>
 *     <prop name="PERSISTENT_UNIT" value="hellojpa"/>
 *     <prop name="debug" value="true"/>
 *     <prop name="PATTERN_IMPL_CLASS" value="com.ibm.websphere.samples.JPAOutputStream"/>
 *   </bds>
 *  
 * 
 * 
 */
public interface JPAWriterPattern {

	/**
	 * This method is invoked during create job step to allow the JPAWriter stream to
	 * initialize. 
	 * @param props Properties passed via xJCL 
	 */
	public void initialize(Properties props);
	
	/**
	 * This method is invoked to actually persist the passed object to the database
	 * using JPA EntityManager 
	 * @param manager
	 * @param record
	 */
	public void writeRecord(EntityManager manager, Object record);
}
