// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

import com.ibm.websphere.batch.BatchContainerDataStreamException;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;

public class LocalJDBCConnection implements IJDBCConnection {
	
	protected String jdbc_url = "jdbc:derby:C:\\db-derby-10.3.1.4-bin\\bin\\firstdb";
	protected String jdbc_driver = "org.apache.derby.jdbc.EmbeddedDriver";
	protected String userid = null;
	protected String pswd = null;
	protected BDSFWLogger logger;
	private boolean autoCommitEnabled = true;
	

	private Connection conn;

	public void commit() throws SQLException {
		conn.commit();
	}
	public void close() throws BatchContainerDataStreamException {
		try {
			conn.close();
		} catch (SQLException e) {
			
			e.printStackTrace();
			throw new BatchContainerDataStreamException(e);
		}

	}

	public Connection getJDBCConnection() {
		return conn;
	}

	public void initialize(Properties props) {
		logger = new BDSFWLogger(props);
		jdbc_url = getRequiredProperty(props,BDSFrameworkConstants.JDBC_URL_KEY);
		
		jdbc_driver = getRequiredProperty(props,BDSFrameworkConstants.JDBC_DRIVER_KEY);
		
		String prop = props.getProperty(BDSFrameworkConstants.USER_ID_KEY);
		if (prop != null) {
			userid = prop.trim();
		}
		prop = props.getProperty(BDSFrameworkConstants.PSWD_KEY);
		if (prop != null) {
			pswd = prop.trim();
		}	
		String autoCommitProp = props.getProperty(BDSFrameworkConstants.AUTO_COMMIT_ENABLED_KEY,"true");
		if(!autoCommitProp.equals("true")) {
			autoCommitEnabled = false;
		}
		
	}

	public void open() throws BatchContainerDataStreamException {
		try {			
            Class.forName(jdbc_driver);        
            if ((userid == null) && (pswd == null))
            {	
            	conn = DriverManager.getConnection(jdbc_url);
            	
            } else {
            	conn = DriverManager.getConnection(jdbc_url, userid, pswd); 
            	
            }
            
            if(!autoCommitEnabled) {
				//disable auto commit 
				conn.setAutoCommit(false);
			}
            if(logger.isDebugEnabled())
        		logger.info("Obtained connection using url:" + jdbc_url);
		}
		catch(ClassNotFoundException aException) {
			logger.error(aException.getLocalizedMessage());
            throw new RuntimeException("driver class not found!", aException);
        }
        catch (Throwable t)
        {
        	logger.error(t.getLocalizedMessage());
        	throw new RuntimeException(t);
        }

	}
	protected String getRequiredProperty(Properties props,String key) {
        String value = props.getProperty(key);
        if (value == null) {
            throw new RuntimeException("Missing required property '" + key + "'");
        }
        value = value.trim();
        if (value.length() == 0) {
            throw new RuntimeException("Required property '" + key + "' has empty value");
        }
        return value;
    }   

}
