// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
/**
 * 
 * @author Snehal/Neeraj
 * 
 * This pattern is used to implement the business logic of processing data from
 *  Z dataset. The administrative tasks of opening, closing the connection
 * is taken care of by the batch framework.
 * 
 * Prequisites:
 * data set name DSNAME,ds_parameters 
 * 
 * E.g
 *  <prop name="DSNAME" value="USER216.BATCH.RECORD.INPUT"/>
 *  
 *  For Text or Byte readers
 *  <prop name="ds_parameters" value=" rt"/>
 *  
 *  for Data reader
 *  <prop name="ds_parameters" value="rb,recfm=fb,type=record,lrecl=80"/>
 *  
 */

package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.io.IOException;
import java.util.Properties;

import com.ibm.jzos.ZFile;



public interface RecordOrientedDatasetReaderPattern {

	/**
	 * This method is invoked during the job setup phase.
	 * The properties are the ones specified in the xJCL.
	 * @param props
	 */
	public void initialize(Properties props);
	
	/**
	 * This method is invoked only once immediately after
	 * the Zfile is opened. It should be used to process
	 * header information if any.
	 * @param reader
	 * @throws IOException
	 */
	public void processHeader(ZFile reader) throws IOException;
	
	/**
	 * This method should read the next record from the Zfile
	 * and return it in an appropriate form (as an intermediate object)
	 * @param reader
	 * @return
	 * @throws IOException
	 */
	public Object fetchRecord(ZFile reader) throws IOException;
	
	/**
	 * Returns the header data that was obtained during the processHeader
	 * call
	 * @return
	 */
	
	public Object fetchHeader();
}
