// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008

/**
 * 
 * @author Snehal/Neeraj
 * 
 * This pattern is used to implement the business logic of outputing data to
 *  Z dataset. The administrative tasks of opening, closing the connection
 * is taken care of by the batch framework.
 * 
 * Prequisites:
 * data set name DSNAME,ds_parameters 
 * 
 * E.g
 *  <prop name="DSNAME" value="USER216.BATCH.RECORD.OUTPUT"/>
 *  
 *  For Text or Byte writers
 *  <prop name="ds_parameters" value=" wt"/>
 *  
 *  for Data writer
 *  <prop name="ds_parameters" value="wb,recfm=fb,type=record,lrecl=80"/>
 *  
 */
package com.ibm.websphere.batch.devframework.datastreams.patternadapter;

import java.io.IOException;
import java.util.Properties;

import com.ibm.jzos.ZFile;

/**
 * 
 * @author Neeraj
 * This pattern is used to write data to zOS dataset using
 * jzos apis
 */
public interface RecordOrientedDatasetWriterPattern {

	/**
	 * This method is called during the job setup phase allowing
	 * the user to do initialization.
	 * The properties are the ones passed in the xJCL
	 * @param props
	 */
	public void initialize(Properties props);
	
	/**
	 * This method should be used to write the given
	 * object into the dataset
	 * @param out
	 * @param record
	 * @throws IOException
	 */
	public void writeRecord(ZFile out, Object record) throws IOException;
	
	/**
	 * This method should be used to write header information
	 * if any
	 * @param out
	 * @throws IOException
	 */
	public void writeHeader(ZFile out) throws IOException;
	
	/**
	 * This method can be optionally called during process step to explicity 
	 * initialize and write the header. 
	 * @param header
	 */
	public void writeHeader(ZFile out, Object header);
	
}
