// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008

package com.ibm.websphere.batch.devframework.datastreams.patterns;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.nio.channels.FileChannel;
import java.util.Properties;

import com.ibm.websphere.batch.BatchContainerDataStreamException;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.BDSFileByteReader;

public class FileByteReader extends BDSFileByteReader {

	public final String RECORDBYTESIZE = "RECORDBYTESIZE";
	protected int recordByteSize;
	protected FileChannel byteReaderFileChannel;

	/**
	 * If the AppendJobIdToFileName property is set to true then the jobid is
	 * appended to the file name
	 */
	public void initialize(Properties props) {

		//this.fileName = props.getProperty(BDSFrameworkConstants.FILENAME_KEY);
		this.fileName = getRequiredProperty(props,BDSFrameworkConstants.FILENAME_KEY);
		String appendJobIdProp = (String)props.get(BDSFrameworkConstants.APPEND_JOBID_TO_FILE_NAME_KEY);
		if (appendJobIdProp != null && appendJobIdProp.equalsIgnoreCase("true"))
		{						
			String jobStepId = props.get(BDSFrameworkConstants.JOB_STEP_ID).toString();
			
			String jobId = jobStepId.substring(0, jobStepId.indexOf('/'));
			
			jobId = jobId.replace('/', '.');
			jobId = jobId.replace(':', '.');
			jobId = jobId.replace('$', '.');
			fileName += "." + jobId;			
		}
		super.initialize(props);
	}
	
	/**
	 * Opens the file and creates a BufferedInputStream. It then invokes
	 * ByteReaderPattern implementation class to process any header information
	 */
	public void open() throws BatchContainerDataStreamException {
		try
		{
			FileInputStream fis = new FileInputStream(fileName);
			byteReaderFileChannel = fis.getChannel();			
			bufferedByteReader = new BufferedInputStream(fis);			
			byteReaderImpl.processHeader(bufferedByteReader);
		}
		catch (Throwable t)
		{
			throw new BatchContainerDataStreamException(t);
		}
	}

	
}
