// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
package com.ibm.websphere.batch.devframework.datastreams.patterns;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.util.Properties;

import com.ibm.websphere.batch.BatchContainerDataStreamException;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.BDSFileReader;

public class TextFileReader extends BDSFileReader {

	public static final String CLASSNAME = TextFileReader.class.getName();
	
	/**
	 * If the AppendJobIdToFileName property is specified and set to true then the job id
	 * is appended to the file name
	 */
	public void initialize(Properties props) {
		logger = new BDSFWLogger(props);

		this.fileName = getRequiredProperty(props,BDSFrameworkConstants.FILENAME_KEY);
		String appendJobIdProp = (String)props.get(BDSFrameworkConstants.APPEND_JOBID_TO_FILE_NAME_KEY);
		if (appendJobIdProp != null && appendJobIdProp.equalsIgnoreCase("true"))
		{						
			String jobStepId = props.get(BDSFrameworkConstants.JOB_STEP_ID).toString();
			
			String jobId = jobStepId.substring(0, jobStepId.indexOf('/'));
			//System.out.println("SSA: jobid= " + jobid);
			jobId = jobId.replace('/', '.');
			jobId = jobId.replace(':', '.');
			jobId = jobId.replace('$', '.');
			fileName += "." + jobId;			
			//if(logger.isDebugEnabled())
			//	logger.debug("Appended Job ID to file name = " + fileName);
		}
		super.initialize(props);
		
		
		
	}

	/**
	 * Opens the file and initializes the BufferedReader. Then it invokes
	 * processHeader on the FileReaderPattern implementation class
	 */
	public void open() throws BatchContainerDataStreamException
	{		
		String methodName = "open";
		if(perfAnalyzer.isEnabled())
			perfAnalyzer.startMeasurement(methodName);
		
		try {
			if (this.fileEncoding != null)
			{
				reader = new BufferedReader(new InputStreamReader(new FileInputStream(fileName), fileEncoding));
			}
			else
			{	
				reader = new BufferedReader(new FileReader(fileName));
			}
			
			fileReaderImpl.processHeader(reader);
			if(logger.isDebugEnabled())
				logger.debug(CLASSNAME + "." + methodName + "Opened file");
		} catch (Throwable t) {
			throw new BatchContainerDataStreamException(t);			
		}
		if(perfAnalyzer.isEnabled())
			perfAnalyzer.endMeasurementAndSave(methodName);
	}
}
