// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
package com.ibm.websphere.batch.devframework.datastreams.patterns;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.Properties;

import com.ibm.jzos.ZFile;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.BDSFileByteReader;

public class ZFileStreamOrientedByteReader extends BDSFileByteReader {
	public static final String CLASSNAME = ZFileStreamOrientedByteReader.class.getName(); 

	protected String dsname = "";
	protected String ds_parameters = "rt";
	protected ZFile zFile;	
	protected BDSFWLogger logger;


	public void initialize(Properties props) {
		
		logger = new BDSFWLogger(props);
		dsname = getRequiredProperty(props,BDSFrameworkConstants.DSNAME_KEY);
		//if (prop != null)
		//{
		//	this.dsname = prop;
		//}
		
		String prop = props.getProperty(BDSFrameworkConstants.DSParameters_KEY);
		if (prop != null)
		{
			this.ds_parameters = prop;
		}
		super.initialize(props);
	}

	public void open() {
		String methodName = "open";
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		try
		{			
			//zFile = new ZFile("//'" + dsname + "'", "rb,type=record,recfm=" + recfm + ",lrecl=" + lrecl +",blksize=" + blksize);
			zFile = new ZFile("//'" + dsname + "'", ds_parameters);     			
			bufferedByteReader = new BufferedInputStream(zFile.getInputStream()); 	
			
			if (logger.isDebugEnabled())
				logger.debug("zFileStreamOrientedByteReader- bufferedByteReader.available= " + bufferedByteReader.available());
			
			if (logger.isDebugEnabled())
				logger.debug("zFileStreamOrientedByteReader- zfile.getInputStream.available= " + zFile.getInputStream().available());
			
			this.byteReaderImpl.processHeader(bufferedByteReader);
		}
		catch (Throwable t)
		{
			throw new RuntimeException(t);
		}     
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
	}

	public boolean hasNext()  throws IOException {
		String methodName = "hasNext";
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);

		boolean retval = false;
		// Did I prefetch a record in the previous read?

		// I already have a value set to the record object.  
		if (record != null) {
			// Have I already processed this value?
			if (prefetchedRecordHasBeenRead) {
				// yes I have. go get another one.
				record = read();

				// was there any data left?
				if (record == null) {
					// no, there was no data left.
					retval = false;
				} else {
					// yes, there was data left. We've therefore prefetched the data.
					prefetchedRecordHasBeenRead = false;
					retval = true;
				}
			}
		} else {
			// I don't have data prefetched. This could be the first read we hit. Go get a record.
			record = read();

			// was there any data left?
			if (record == null) {
				// no, there wasn't any data left. 
				retval = false;
			} else {
				// yes, there was data left. We've therefore just prefetched it. 
				retval = true;
				prefetchedRecordHasBeenRead = false;
			}
		}
		if (logger.isDebugEnabled())
			logger.debug(CLASSNAME + "." + methodName +"="+ retval);
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
		return retval;
	}

	public Object read()  throws IOException {
		String methodName = "read";
		Object returnValue = null;
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		recordCount++;
		if (!prefetchedRecordHasBeenRead) {
			if (record != null) {
				prefetchedRecordHasBeenRead = true;
				returnValue = record;
			} else {
				returnValue = byteReaderImpl.fetchRecord(bufferedByteReader);
			}	
		} else {
			returnValue = byteReaderImpl.fetchRecord(bufferedByteReader);
		}
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
		return returnValue;
	}

}
