// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008

package com.ibm.websphere.batch.devframework.datastreams.patterns;

import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.util.Properties;

import com.ibm.jzos.ZFile;
import com.ibm.websphere.batch.BatchContainerDataStreamException;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.BDSFileWriter;

public class ZFileStreamOrientedTextWriter extends BDSFileWriter {
	public static final String CLASSNAME = ZFileStreamOrientedTextWriter.class.getName();

	protected String dsname = "";
	protected String ds_parameters = "wt";
	protected ZFile zFile;
	protected String defaultFileEncoding = "CP1047";
	
	
	public void initialize(Properties props) {		
		dsname = getRequiredProperty(props,BDSFrameworkConstants.DSNAME_KEY);
		//if (prop != null)
		//{
		//	this.dsname = prop;
		//}
		
		String prop = props.getProperty(BDSFrameworkConstants.DSParameters_KEY);
		if (prop != null)
		{
			this.ds_parameters = prop;
		}
		
		super.initialize(props);
	}

	public void close() throws BatchContainerDataStreamException {
		try
		{
			writer.close();
			if(isPerformanceMeasurementEnabled)
				perfAnalyzer.displayPerformanceStatisticsForEntity();
		}
		catch (Throwable t)
		{
			throw new BatchContainerDataStreamException(t);
		}
	}
	
	public void setPosition(long position) {
		try {
			zFile.seek(position,0);
		} 
		catch (Throwable t) {			
			throw new RuntimeException("Failed to set position of zFile", t);
		}
	}

	public long getCurrentPosition() {
		try
		{
			writer.flush();
			return zFile.tell();
		}
		catch (Throwable t)
		{
			throw new RuntimeException("Failed to retrieve the current position of the zFile.", t);
		}
	}

	public void open() throws BatchContainerDataStreamException {		
		String methodName = "open";
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
        try {
            // Try to open existing dataset.
            //zFile = new ZFile("//'" + dsname + "'", "r+b" + modeString);
        	zFile = new ZFile("//'" + dsname + "'", ds_parameters);   
        	
        	if (this.fileEncoding != null)
        	{
        		if (logger.isDebugEnabled())
        			logger.debug("ZFileStreamOrientedTextWriter.open(). File.encoding= " + fileEncoding);
        		
        		writer = new BufferedWriter(new OutputStreamWriter(zFile.getOutputStream(), fileEncoding));
        	}
        	else
        	{
        		if (logger.isDebugEnabled())
        			logger.debug("ZFileStreamOrientedTextWriter.open(). Default File.encoding= " + defaultFileEncoding);
        		writer = new BufferedWriter(new OutputStreamWriter(zFile.getOutputStream(), this.defaultFileEncoding));
        	}                    	
        } catch (Throwable t) {
            throw new BatchContainerDataStreamException(t);
        }		
        if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
	}

	
}
