// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
package com.ibm.websphere.batch.devframework.test;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

public class IsIdenticalTask extends Task {
	
	private String fromFile, toFile, diffFile;
	private static boolean failuresOccured = false;
	
	public String getDiffFile() {
		return diffFile;
	}



	public void setDiffFile(String resultsFile) {
		this.diffFile = resultsFile;
	}



	public void execute() throws BuildException 
	{
		BufferedReader fileReader1, fileReader2 = null;
		try {
			
			fileReader1 = _openFileToRead(fromFile);
			fileReader2 = _openFileToRead(toFile);
			
			boolean result = _compareFiles(fileReader1, fileReader2);
			
			fileReader1.close();
			fileReader2.close();
			
			_outputResult(result, fromFile, toFile, diffFile);
			
			if (failuresOccured)
				throw new BuildException("Atleast 1 failure occured please check the diff folder");
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new BuildException(e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new BuildException(e);
		}
		
		
	}

	
	
	private void _outputResult(boolean result, String fromFileName,
			String toFileName, String resultsFile) throws IOException 
	{
		BufferedWriter fileWriter = null;
		 
		fileWriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(resultsFile)));
		
		if( result )
			fileWriter.write(fromFileName + " EQUAL TO " + toFileName);
		else
		{	
			fileWriter.write(fromFileName + " NOT EQUAL TO " + toFileName);
			System.out.println(fromFileName + " NOT EQUAL TO " + toFileName);
			failuresOccured = true;
		}	
		fileWriter.close();
		
	}



	private BufferedReader _openFileToRead(String file) throws FileNotFoundException
	{
		BufferedReader fileReader = null;
	 
		fileReader = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
		
		return fileReader;
	}
	
	private boolean _compareFiles(BufferedReader fromFileRd, BufferedReader toFileRd) 
		throws IOException
	{
		boolean isIdentical = true;
		
		while(fromFileRd.ready() && toFileRd.ready())
		{
			String str1 = fromFileRd.readLine();
			String str2 = toFileRd.readLine();
			if (!str1.equals(str2))
			{
				isIdentical = false;
				break;
			}
			
		}
		if(fromFileRd.ready() || toFileRd.ready())
		{
			isIdentical = false;
		}
		return isIdentical;
	}
	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {

		if (args.length == 3)
		{	
			IsIdenticalTask fd = new IsIdenticalTask();
			fd.setFromFile(args[0]);
			fd.setToFile(args[1]);
			fd.setDiffFile(args[2]);
			fd.execute();
			
		}
		else
		{
			System.out.println("Usage: java com.ibm.websphere.batch.devframework.test.FileDiff " +
					"<file1> <file2> <file3");
		}
		

	}

	public String getFromFile() {
		return fromFile;
	}

	public void setFromFile(String fromFileName) {
		this.fromFile = fromFileName;
	}

	public String getToFile() {
		return toFile;
	}

	public void setToFile(String toFileName) {
		this.toFile = toFileName;
	}

}
