// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008


/**
 * This interface is used to provide a threshold policy implementation to be used
 * in conjunction with the Threshold BatchStep
 */
package com.ibm.websphere.batch.devframework.thresholdpolicies;

import java.util.Properties;

public interface ThresholdPolicy {
	/**
	 * Constant indicating threshold violation has not occured
	 */
	public static int NO_THRESHOLD_VIOLATION = 0;
	
	/**
	 * Constant indicating threshold violation has occured.
	 */
	public static int THRESHOLD_VIOLATION = 1;
	/**
	 * This method is invoked during StepSetup allows any configuration
	 * properties passed via xJCL to be read
	 * @param props
	 */
	public void initialize(Properties props);
	
	/**
	 * This method is invoked by the ThresholdBatchStep each time the BatchRecordProcessor
	 * returns a null value. This method allows the user to calculate how many errors have 
	 * been encountered till now and whether they exceed the threshold.
	 * @param currentRecordCount: The count of the current record being processed
	 * @return
	 */
	public int invalidRecordEncountered(int currentRecordCount);
	
	/**
	 * Called during step shutdown allows any cleanup to occur
	 */
	public void destroy();

}
