// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
package com.ibm.ws.batch;


import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import javax.ejb.EJBException;

import com.ibm.websphere.batch.BatchContainerDataStreamException;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.IJDBCConnection;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.JDBCConnection;

/**
 * Bean implementation class for Enterprise Bean: CursorHoldableJDBCReader
 */
public class CursorHoldableJDBCReaderBean	implements	javax.ejb.SessionBean {

	static final long serialVersionUID = 3206093459760846163L;
	private javax.ejb.SessionContext mySessionCtx;
	
	private IJDBCConnection jdbcConn;
	private ResultSet resultSet;
	private PreparedStatement pstmt = null;
	
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
	public ResultSet getNext() {
		return resultSet;
	}
	public void initialize(Properties props) {
		
		try 
		{
			
			if(jdbcConn == null)
			{	
				jdbcConn = new JDBCConnection();
				props.put(BDSFrameworkConstants.CURSOR_HOLDABILITY_ENABLED, "true");
				jdbcConn.initialize(props);
			}	
			
		
			
		}
		catch (Throwable t)
		{
			throw new RuntimeException(t);
		}
		 
	}
	
	private void _positionResultSet(String query)  {
		try {
			
				pstmt = jdbcConn.getJDBCConnection().prepareStatement(query);
				resultSet = pstmt.executeQuery();
				
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new EJBException(e);
		}
		
	}
	
	public void positionAtInitialCheckpoint(String query) {
		
		_positionResultSet(query);
			
	}
	public void positionAtCurrentCheckpoint(String restartQuery){
		_positionResultSet(restartQuery);
		
	}
	public Object get(Object sql) {	
		
		Statement st = null;
		Object result = null;
	
		try {
			st = jdbcConn.getJDBCConnection().createStatement();
			result  =  st.executeQuery((String)sql);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new EJBException(e);
		}
		
		
		return result;
	
	}

	public boolean hasNext()  {
		boolean hasNxt = false;
		
		try {
			hasNxt = resultSet.next();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new EJBException(e);
		}
		
		return hasNxt;
	}	
	public void open() {
		try {
			jdbcConn.open();
		} catch (BatchContainerDataStreamException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new EJBException(e);
		}
				
	}
	public void close(){
		try {
			if(pstmt!=null) {
				pstmt.close();
			}
			
			jdbcConn.close();
		} catch (BatchContainerDataStreamException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new EJBException(e);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new EJBException(e);
		}
				
	}
	
	public void internalizeCheckpoint(String query) {
		_positionResultSet(query);
	}

}
